@@ -1,4 +1,4 @@
-# $Id: Bug.pm,v 1.40 2001/12/01 15:24:42 richardf Exp $
+# $Id: Bug.pm,v 1.43 2002/01/14 10:14:48 richardf Exp $
 #
 
 =head1 NAME
@@ -10,7 +10,7 @@ Perlbug::Object::Bug - Bug class
 package Perlbug::Object::Bug;
 use strict;
 use vars qw($VERSION @ISA);
-$VERSION = do { my @r = (q$Revision: 1.40 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; 
+$VERSION = do { my @r = (q$Revision: 1.43 $ =~ /\d+/g); sprintf "%d."."%02d" x $#r, @r }; 
 $|=1;
 
 my %fmt = ();
@@ -62,6 +62,7 @@ sub new {
 	my $self = Perlbug::Object->new( $o_base, 
 		'name'		=> 'Bug',
 		'match_oid'	=> '(\d{8}\.\d{3})',
+		'match_name'=> '(\d{8}\.\d{3})',
 		'from'		=> [qw(group project range user)],
 		'to'		=> [qw(
 			address change child fixed message note osname parent patch severity status test version
@@ -72,62 +73,57 @@ sub new {
 }
 
 
-sub read {
-	my $self = shift;
-	$self->SUPER::read(@_);
-	return $self;  
-}
-
-
-=item update
-
-extend B<SUPER::update> with notify_cc (of changes)
-
-=cut
-
-sub xupdate {
-	my $self = shift;
-
-	# my $ix = $self->notify_cc($bid, '', $orig) unless $nocc eq 'nocc';
-
-	return $self->SUPER::update(@_);
-}
-
-
 =item new_id
 
-Return valid new object id for bug, wrapper for base->new_id
+Generate new_id for perlbug bug
 
-	my $new_oid = $o_bug->new_id
-
-# =cut redundant
-
-sub xnew_id {
-	my $self = shift;
-
-	my $newid = $self->base->new_id;	
-
-	return $newid;
-}
-
-
-=item insertid
-
-Returns newly inserted id from recently created object.
-
-	my $new_oid = $o_obj->insertid();
+	my $new_id = $o_bug->new_id;
 
 =cut
 
-sub xinsertid {
-	my $self = shift;
-	my $sth  = shift;		# ignored
-	my $oid  = shift || 'unspecified'; 
+sub new_id {
+    my $self  = shift;
 
-	# my $oid = $self->data($self->attr('primary_key'));	
-	$self->debug(1, "newly inserted bugoid($oid)") if $Perlbug::DEBUG;
+    my $today  = $self->base->get_date();
+	my $newid  = "$today.001";
+	my @extant = ($self->base->get_list("SELECT max(bugid) FROM pb_bug"), $self->base->extant);
+	my ($max)  = sort { $b <=> $a } @extant;
+	if ($max =~ /^(\d{8})\.(\d{3})$/o) {
+		my $num = ($1 eq $today) ? $2 + 1 : 1;
+		$newid = $today.'.'.sprintf("%03d", $num);
+		if (grep(/^$newid$/, @extant)) {
+			$newid = $today.'.'.sprintf("%03d", $num + 1);
+		} # parent/child fix
+		if ($num >= 999) {
+			$self->error("Ran out of bug ids today ($today) at: '$newid'");
+		}
+	}
+	$self->debug(1, "today($today), extant(@extant) max($max) => newid($newid)") if $Perlbug::DEBUG;
+	$self->base->extant($newid);
 
-	return $oid; 
+    return $newid;
+}
+
+
+=item get_id
+
+Determine if the string contains a valid bug ID.
+
+    my ($ok, $tid) = $obj->get_id($str);
+
+=cut
+
+sub get_id {
+    my $self = shift;
+    my $str = shift;
+    my ($ok, $id) = (0, '');
+    # /^\[[ID]*\s*(\d{8}\.\d{3})\s*\]$/ -> brackets ...?
+    if ($str =~ /(\d{8}\.\d{3})/o) { # no \b while _ is a letter?
+        $id = $1;
+        $ok = 1;
+    }
+    $self->debug(3, "str($str) -> $ok ($id)") if $Perlbug::DEBUG;
+    return ($ok, $id);
 }
 
 
@@ -156,11 +152,14 @@ sub htmlify {
 	# messages
     my @mids = (ref($bug{'message_ids'}) eq 'ARRAY') ? @{$bug{'message_ids'}} : ($bug{'message_ids'});
 	$bug{'message_ids'} = \@mids;
-    my $cnt = @mids;
-    my $msgs = (@mids == 1) ? "($cnt msg)" : "($cnt msgs)";
-	my $stat_msgs = "$cnt messages for bug($bid)";
-    my ($allmsgs) = $self->href('bidmids', [$bid], $msgs, $stat_msgs);
- 
+	my $allmsgs = 'no msgs';
+	if (scalar(@mids) >= 1) {
+		my $cnt = @mids;
+		my $msgs = (@mids == 1) ? "($cnt msg)" : "($cnt msgs)";
+		my $stat_msgs = "$cnt messages for bug($bid)";
+		($allmsgs) = $self->href('bidmid', [$bid], $msgs, $stat_msgs);
+	}
+
 	%bug = %{$self->SUPER::htmlify($h_bug)};	
 	$bug{'bugid'}     =~ s/format=h/format=H/;
     $bug{'bugid'}    .= " &nbsp; $allmsgs";
@@ -196,7 +195,7 @@ sub htmlify {
 		$bug{'address_names'} = $self->object('address')->text_field($bid.'_address', '', -'size' => 55).$bug{'address_ids'};
 		$bug{'note_names'}   	= $self->object('note')->text_field($bid.'_note', '').$bug{'note_ids'};
 		$bug{'group_names'}  	= $self->object('group')->choice($bid.'_group', @groups); 
-		$bug{'change_names'}  = $self->object('change')->text_field($bid.'_change', '').$bug{'change_ids'};
+		$bug{'change_names'}  = $self->object('change')->text_field($bid.'_change', '').$bug{'change_names'};
 		$bug{'child_ids'}   = $self->object('child')->text_field($bid.'_child', '').$bug{'child_ids'};
         $bug{'fixed_names'} = $self->object('fixed')->text_field($bid.'_fixed', $fixed);
 		# new stuff is only for format::H
@@ -211,7 +210,7 @@ sub htmlify {
 		$bug{'test_ids'}    = $self->object('test')->text_field($bid.'_test', '').$bug{'test_ids'};
 		$bug{'severity_names'}= $self->object('severity')->choice($bid.'_severity', $severity);
         $bug{'status_names'}  = $self->object('status')->choice($bid.'_status', $status);
-    	$bug{'select'}      = $cgi->checkbox(-'name'=>'bugids', -'checked' => '', -'value'=> $bid, -'label' => '', -'override' => 1);
+    	$bug{'select'}      = $cgi->checkbox(-'name'=>'bugid', -'checked' => '', -'value'=> $bid, -'label' => '', -'override' => 1);
         # $bug{'user_ids'}  = $self->object('user')->choice($bid.'_user', @users);
         $bug{'version_names'} = $self->object('version')->text_field($bid.'_version', $version);
 	}
@@ -220,6 +219,105 @@ sub htmlify {
 	return \%bug;
 }
 
+
+=item webupdate
+
+Update bug via web interface
+
+	my $oid = $o_bug->webupdate(\%cgidata, $gid);
+
+=cut
+
+sub webupdate {
+	my $self   = shift;
+	my $h_data = shift;
+	my $oid    = shift;
+    my $cgi    = $self->base->cgi();
+
+	if (!(ref($h_data) eq 'HASH')) {
+		$self->error("requires data hash ref($h_data) to update Bug data via the web!");
+	} else {
+		if (!($self->ok_ids([$oid]))) {
+			$self->error("No bugid($oid) for webupdate!".Dumper($h_data));
+		} else {
+			my $orig_fmt = $self->base->current('format');
+			$self->read($oid);
+			if ($self->READ) {
+				$self->base->current({'context', 'text'}); # notify_cc
+				$self->base->current({'format',  'a'});
+
+				my $i_read = $self->read($oid)->READ;
+				my $orig = $self->format('a');	
+				$self->base->dok([$oid]);
+
+				my $opts = $cgi->param($oid.'_opts') || $cgi->param('_opts') || '';
+				my $pars = join(' ', $opts);
+				my %update = $self->base->parse_str($pars);
+
+				REL: # space separated(str2ids), store/assign(friendly/prejudicial)
+				foreach my $rel ($self->rels) { 			# rels
+					my @extant = $self->rel_ids($rel);
+					push(@{$update{$rel}{'ids'}}, @extant) if scalar(@extant) >= 1;
+					my @update = ($rel =~ /(change|patch|note|test|parent|child)/io) 
+						? split(/\s+/, $cgi->param($oid.'_'.$rel))  # space seperated
+						: $cgi->param($oid."_$rel");				# plain
+					my $type = ($rel =~ /(address|change|fixed|version)/) 
+						? 'names' : 'ids';
+					push(@{$update{$rel}{$type}}, @update) if scalar(@update) >= 1;
+					my %data = (
+						'rel'		=> $rel, 
+						'type'		=> $type,
+						'update'	=> \@update,
+						'extant'	=> \@extant,
+					);
+					$self->debug(1, Dumper(\%data)) if $Perlbug::DEBUG;
+				}				
+				my $i_rel = $self->relate(\%update);
+
+				my $req = $cgi->param('req') || '';
+				if ($self->base->current('mailing') == 1) {
+					my $ix = $self->base->notify_cc($oid, $orig) unless $req eq 'nocc'; 
+				}
+
+				foreach my $targ (qw(note patch test)) { 	# new
+					my $call = 'do'.uc(substr($targ, 0, 1));
+					my $i_newid  = $self->base->$call({
+						'opts'	=> "req($req): $oid", 
+						'body'	=> $cgi->param($oid.'_new_'.$targ),
+					}) if $cgi->param($oid.'_new_'.$targ);
+				}
+				my $ref = "<p>Bug ($oid) updated $Mysql::db_errstr<p>";
+				$self->debug(2, $ref) if $Perlbug::DEBUG;
+
+				$self->base->current({'context', 'html'});
+				$self->base->current({'format', $orig_fmt});
+			}
+		}
+	}
+	
+	return $oid;
+}
+
+
+=pod
+
+=back
+
+=head1 AUTHOR
+
+Richard Foley perlbug@rfi.net 2000
+
+=cut
+
+# 
+1;
+
+# ================================================================================ 
+
+__END__
+
+REDUNDANT from here down!
+
 =pod
 
 =back
@@ -534,69 +632,3 @@ $$x{'group_names'} &nbsp;</td>
 }
 
 
-=item new_id
-
-Generate new_id for perlbug bug
-
-	my $new_id = $o_bug->new_id;
-
-=cut
-
-sub new_id {
-    my $self  = shift;
-
-    my $today  = $self->base->get_date();
-	my $newid  = "$today.001";
-	my @extant = ($self->base->get_list("SELECT max(bugid) FROM pb_bug"), $self->base->extant);
-	my ($max)  = sort { $b <=> $a } @extant;
-	if ($max =~ /^(\d{8})\.(\d{3})$/o) {
-		my $num = ($1 eq $today) ? $2 + 1 : 1;
-		$newid = $today.'.'.sprintf("%03d", $num);
-		if (grep(/^$newid$/, @extant)) {
-			$newid = $today.'.'.sprintf("%03d", $num + 1);
-		} # parent/child fix
-		if ($num >= 999) {
-			$self->error("Ran out of bug ids today ($today) at: '$newid'");
-		}
-	}
-	$self->debug(1, "today($today), extant(@extant) max($max) => newid($newid)") if $Perlbug::DEBUG;
-	$self->base->extant($newid);
-
-    return $newid;
-}
-
-
-=item get_id
-
-Determine if the string contains a valid bug ID.
-
-    my ($ok, $tid) = $obj->get_id($str);
-
-=cut
-
-sub get_id {
-    my $self = shift;
-    my $str = shift;
-    my ($ok, $id) = (0, '');
-    # /^\[[ID]*\s*(\d{8}\.\d{3})\s*\]$/ -> brackets ...?
-    if ($str =~ /(\d{8}\.\d{3})/o) { # no \b while _ is a letter?
-        $id = $1;
-        $ok = 1;
-    }
-    $self->debug(3, "str($str) -> $ok ($id)") if $Perlbug::DEBUG;
-    return ($ok, $id);
-}
-
-=pod
-
-=back
-
-=head1 AUTHOR
-
-Richard Foley perlbug@rfi.net 2000
-
-=cut
-
-# 
-1;
-
